﻿Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.Civil.ApplicationServices
Imports Autodesk.Civil.Land.DatabaseServices

Public Class Class1
    <CommandMethod("PSCRALIGN")> _
    Public Sub CreateAlignmentFromPLine()

        '' This sample Demonstrates Creation of an Alignment in Civil 3D 2011 using the new .NET API - 
        '' User need to select a Polyline which will be converted to an Aligne
        '' Created by Partha P. Sarkar - DevTech, Autodesk

        Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor

        ' Select the Polyline
        Dim promptEntOp As New PromptEntityOptions(vbCrLf + "Select a Polyline : ")
        Dim promptEntRs As PromptEntityResult
        promptEntRs = ed.GetEntity(promptEntOp)
        If promptEntRs.Status <> PromptStatus.OK Then
            ed.WriteMessage("Exiting! Try Again !")
            Exit Sub
        End If
        Dim idEnt As ObjectId
        idEnt = promptEntRs.ObjectId

        Dim trans As Transaction = HostApplicationServices.WorkingDatabase.TransactionManager.StartTransaction()
        Dim civilDoc As CivilDocument = Autodesk.Civil.ApplicationServices.CivilApplication.ActiveDocument

        Try

            Dim alignId As ObjectId = Nothing
            Dim poptions As Autodesk.Civil.Land.DatabaseServices.PolylineOptions
            poptions.AddCurvesBetweenTangents = True
            poptions.EraseExistingEntities = True
            poptions.PlineId = idEnt
            ' Check the Alignment Style Name and labelSetName 
            ' To use the following default names you can use the "Align-6A.dwg " from Help\Civil Tutorials\Drawings\
            ' and make sure to create a empty Site named "Site 1"(if it is already not there in the DWG file), also draw a polyline to convert it to Alignment.

            alignId = Autodesk.Civil.Land.DatabaseServices.Alignment.Create(civilDoc, poptions, "Test_Alignment", "Site 1", "0", "Design Style", "All Labels")

            trans.Commit()
        Catch ex As Exception

            ed.WriteMessage("Exception Message is  : " + ex.Message.ToString())

        Finally
            trans.Dispose()
        End Try
    End Sub

    <CommandMethod("PSCROFFSET")> _
    Public Sub CreateOffsetAlignment()

        '' This sample Demonstrates Creation of an Offset Alignment in Civil 3D 2011 using the new .NET API - 
        '' 
        '' Created by Partha P. Sarkar - DevTech, Autodesk

        Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor

        ' Select the Alignment
        Dim promptEntOp As New PromptEntityOptions(vbCrLf + "Select an Alignment : ")
        Dim promptEntRs As PromptEntityResult
        promptEntRs = ed.GetEntity(promptEntOp)
        If promptEntRs.Status <> PromptStatus.OK Then
            ed.WriteMessage("Exiting! Try Again !")
            Exit Sub
        End If

        Dim trans As Transaction = HostApplicationServices.WorkingDatabase.TransactionManager.StartTransaction()
        Dim civilDoc As CivilDocument = Autodesk.Civil.ApplicationServices.CivilApplication.ActiveDocument

        Try

            Dim alignEnt As Alignment
            'Dim ent As Entity
            Dim ent As Autodesk.Civil.DatabaseServices.Entity
            ent = trans.GetObject(promptEntRs.ObjectId, OpenMode.ForRead)
            If TypeOf (ent) Is Alignment Then

                alignEnt = ent
                If alignEnt Is Nothing Then
                    ed.WriteMessage(vbCrLf + "Selected Entity was Not an Alignment! Try Again !")
                    Exit Sub
                End If

                ' you can use the  "Align-6A.dwg " from Help\Civil Tutorials\Drawings\ 
                Dim offsetAlignId As ObjectId = Alignment.CreateOffsetAlignment("Offset_Alignment", alignEnt.ObjectId, 10, alignEnt.StyleId)
                trans.Commit()
            Else
                ed.WriteMessage(vbCrLf + "Selected Entity was Not an Alignment! Try Again !")
            End If

        Catch ex As Exception

            ed.WriteMessage("Exception Message is  : " + ex.Message.ToString())

        Finally
            trans.Dispose()
        End Try
    End Sub


    <CommandMethod("PSDS")> _
    Public Sub AddDesignSpeed()

        '' This sample Demonstrates adding design speeds along the length of an alignment using .NET API in Civil 3D 2011 /2010 - 
        '' 
        '' Created by Partha P. Sarkar - DevTech, Autodesk

        Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor

        ' Select the Alignment
        Dim promptEntOp As New PromptEntityOptions(vbCrLf + "Select an Alignment : ")
        Dim promptEntRs As PromptEntityResult
        promptEntRs = ed.GetEntity(promptEntOp)
        If promptEntRs.Status <> PromptStatus.OK Then
            ed.WriteMessage("Exiting! Try Again !")
            Exit Sub
        End If

        Dim trans As Transaction = HostApplicationServices.WorkingDatabase.TransactionManager.StartTransaction()
        Dim civilDoc As CivilDocument = Autodesk.Civil.ApplicationServices.CivilApplication.ActiveDocument

        Try

            Dim alignEnt As Alignment
            Dim designSpeed As DesignSpeed = Nothing
            Dim ent As Autodesk.AutoCAD.DatabaseServices.Entity
            ent = trans.GetObject(promptEntRs.ObjectId, OpenMode.ForRead)
            If TypeOf (ent) Is Alignment Then

                alignEnt = ent
                If alignEnt Is Nothing Then
                    ed.WriteMessage(vbCrLf + "Selected Entity was Not an Alignment! Try Again !")
                    Exit Sub
                End If

                ' you can use the  "Align-6A.dwg " from Help\Civil Tutorials\Drawings\ 
                ' lets add Design Speed 60 mph at Station  00.00
                ' This Alignment in Align-6A.dwg has already a Design Speed Assigned for Starting Station
                ' we need to Remove this first

                'alignEnt.DesignSpeeds.Remove(0.0)

                ' Now add a new Design speed for the same station
                designSpeed = alignEnt.DesignSpeeds.Add(0.0, 60.0)
                designSpeed.Comment = "This is a Straightway"


                ' lets add Design Speed 40 mph at Station  300.00

                designSpeed = alignEnt.DesignSpeeds.Add(300.0, 40.0)
                designSpeed.Comment = "Start of curve"

                ' lets add Design Speed 70 mph at Station  501.00

                designSpeed = alignEnt.DesignSpeeds.Add(501.0, 70.0)
                designSpeed.Comment = "End of Curve and Starting of Straightway"

                ' ensure the alignment design is speed-based
                alignEnt.UseDesignSpeed = True
                trans.Commit()
            Else
                ed.WriteMessage(vbCrLf + "Selected Entity was Not an Alignment! Try Again !")
            End If

        Catch ex As Exception

            ed.WriteMessage("Exception Message is  : " + ex.Message.ToString())

        Finally
            trans.Dispose()
        End Try
    End Sub
End Class